@@ -7,13 +7,13 @@ from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
 from ground_soil.fabric import eval_kwargs, virtual_env
-from ground_soil.filesystem import temporary_directory, render_file
+from ground_soil.filesystem import temporary_directory, render_file, sed
 from beanstalk import IDENTIFIER, BEANSTALK_ROOT_PATH
 from beanstalk.decorators import beanstalk_role
 from beanstalk.paths.server import (project_base_path, project_source_path, project_logs_path, project_venv_path,
                                     project_uwsgi_path, apache_conf_path, app_ports_path)
 from beanstalk.tasks.utils import load_role_settings as load_beanstalk_settings, set_verbose_level, run_hooked_actions
-from beanstalk.titles import tool_title, section_title, separator
+from beanstalk.console.titles import tool_title, section_title, separator
 
 
 # Tasks
@@ -22,7 +22,7 @@ from beanstalk.titles import tool_title, section_title, separator
 
 @task
 @beanstalk_role('server')
-def setup(settings_path=None, **settings_patches):
+def setup(**settings_patches):
     """Setup remote base of beanstalk-stack in this server
     """
     print tool_title('Beanstalk-stack server setup tool')
@@ -30,7 +30,7 @@ def setup(settings_path=None, **settings_patches):
 
     # Load beanstalk settings
     settings_patches = eval_kwargs(settings_patches)
-    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    beanstalk_settings = load_beanstalk_settings(**settings_patches)
     beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
     set_verbose_level()
 
@@ -56,22 +56,33 @@ def setup(settings_path=None, **settings_patches):
                 abort('Failed to extract uwsgi source folder')
                 return
 
-            with lcd(uwsgi_src_folder):
-                # Create build configuration
-                local('mkdir -p /usr/local/lib/uwsgi')
-                uwsgi_build_ini_path = os.path.join(BEANSTALK_ROOT_PATH, 'static/uwsgi_build.ini')
-                local('cp %s buildconf/beanstalk.ini' % uwsgi_build_ini_path)
-                # Build and install
-                local('python uwsgiconfig.py --build beanstalk')
-                local('cp uwsgi /usr/local/bin/uwsgi')
-                if not os.path.exists('/usr/bin/uwsgi'):
-                    local('ln -s /usr/local/bin/uwsgi /usr/bin/uwsgi')
-                local('mkdir -p /var/run/beanstalk')
-                local('cp %s /etc/init.d/beanstalk' % os.path.join(BEANSTALK_ROOT_PATH, 'static/beanstalk_init.sh'))
-                local('chmod +x /etc/init.d/beanstalk')
-                # mod_uwsgi
-                local('apxs -i -c apache2/mod_uwsgi.c')
-                local('echo "LoadModule uwsgi_module modules/mod_uwsgi.so" > /etc/httpd/conf.d/uwsgi.conf')
+            # Check version
+            install_uwsgi = True
+            with settings(hide('everything'), warn_only=True):
+                has_uwsgi = local('which uwsgi', capture=True).return_code == 0
+                if has_uwsgi:
+                    new_version = tuple(map(lambda x: int(x), uwsgi_src_folder.split('-')[-1].split('.')))
+                    current_version = tuple(map(lambda x: int(x), local('uwsgi --version', capture=True).split('.')))
+                    if new_version == current_version:
+                        install_uwsgi = confirm('Re-install Uwsgi?', default=False)
+
+            if install_uwsgi:
+                with lcd(uwsgi_src_folder):
+                    # Create build configuration
+                    local('mkdir -p /usr/local/lib/uwsgi')
+                    uwsgi_build_ini_path = os.path.join(BEANSTALK_ROOT_PATH, 'static/uwsgi_build.ini')
+                    local('cp %s buildconf/beanstalk.ini' % uwsgi_build_ini_path)
+                    # Build and install
+                    local('python uwsgiconfig.py --build beanstalk')
+                    local('cp uwsgi /usr/local/bin/uwsgi')
+                    if not os.path.exists('/usr/bin/uwsgi'):
+                        local('ln -s /usr/local/bin/uwsgi /usr/bin/uwsgi')
+                    local('mkdir -p /var/run/beanstalk')
+                    local('cp %s /etc/init.d/beanstalk' % os.path.join(BEANSTALK_ROOT_PATH, 'static/beanstalk_init.sh'))
+                    local('chmod +x /etc/init.d/beanstalk')
+                    # mod_uwsgi
+                    local('apxs -i -c apache2/mod_uwsgi.c')
+                    local('echo "LoadModule uwsgi_module modules/mod_uwsgi.so" > /etc/httpd/conf.d/uwsgi.conf')
 
     # Do you have user "beanstalk"?
     with settings(hide('everything'), warn_only=True):
@@ -99,42 +110,55 @@ def setup(settings_path=None, **settings_patches):
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'apps'))
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'global-logs'))
     local('mkdir -p %s' % os.path.join(beanstalk_stack_base, 'confs'))
-    local('echo "[]" > %s' % app_ports_path())
+    if not os.path.exists(app_ports_path()):
+        local('echo "[]" > %s' % app_ports_path())
     local('chmod o+rx %s' % beanstalk_stack_base)
 
     # Global ini
     local('mkdir -p /etc/beanstalk')
-    global_uwsgi_ini_path = os.path.join(BEANSTALK_ROOT_PATH, 'static/global_uwsgi.ini')
-    with hide('everything'):
-        uid = local('id -u beanstalk', capture=True)
-    global_uwsgi_ini = render_file(global_uwsgi_ini_path, beanstalk_settings, BEANSTALK_UID=uid)
-    with open('/etc/beanstalk/uwsgi.ini', 'w') as f:
-        f.write(global_uwsgi_ini)
+    if not os.path.exists('/etc/beanstalk/uwsgi.ini'):
+        global_uwsgi_ini_template_path = os.path.join(BEANSTALK_ROOT_PATH, 'static/global_uwsgi.ini')
+        with hide('everything'):
+            uid = local('id -u beanstalk', capture=True)
+        global_uwsgi_ini_content = render_file(global_uwsgi_ini_template_path, beanstalk_settings, BEANSTALK_UID=uid)
+        with open('/etc/beanstalk/uwsgi.ini', 'w') as f:
+            f.write(global_uwsgi_ini_content)
 
     # Apache command
     print section_title('Create apache-reload command')
     httpd_cmd_path = '/usr/sbin/httpd_reload'
-    httpd_cmd_source = os.path.join(BEANSTALK_ROOT_PATH, 'static/httpd_reload.sh')
-    local('/bin/cp -f %s %s' % (httpd_cmd_source, httpd_cmd_path))
-    local('chmod +x %s' % httpd_cmd_path)
+    if not os.path.exists(httpd_cmd_path):
+        httpd_cmd_source = os.path.join(BEANSTALK_ROOT_PATH, 'static/httpd_reload.sh')
+        local('/bin/cp -f %s %s' % (httpd_cmd_source, httpd_cmd_path))
+        local('chmod +x %s' % httpd_cmd_path)
 
     # Give beanstalk permission
     print section_title('Give beanstalk permission for apache-reload command')
     sudoer_file = '/etc/sudoers.d/beanstalk_reload_httpd'
-    local('echo "beanstalk ALL=(root) NOPASSWD: %s" > %s' % (httpd_cmd_path, sudoer_file))
-    local('chmod 440 %s' % sudoer_file)
+    if not os.path.exists(sudoer_file):
+        local('echo "beanstalk ALL=(root) NOPASSWD: %s" > %s' % (httpd_cmd_path, sudoer_file))
+        local('chmod 440 %s' % sudoer_file)
 
     # Make beanstalk access ssh
-    local('sed -i.bak -e "/^AllowGroups wheel$/cAllowGroups wheel beanstalk" /etc/ssh/sshd_config')
+    print section_title('Configure SSH server')
+    sshd_config = '/etc/ssh/sshd_config'
+    new_lines = [
+        'Match Group beanstalk',
+        '       ForceCommand /home/beanstalk/check_user',
+    ]
+    local(sed('$a\\{command}'.format(command='\\n'.join(new_lines)), sshd_config))
+    local(sed('s/^AllowGroups (.*)$/AllowGroups \\1 beanstalk/g', sshd_config, '-E'))
 
     # Apache conf
-    beanstalk_apache_conf_template = os.path.join(BEANSTALK_ROOT_PATH, 'static/beanstalk_apache.conf')
-    beanstalk_apache_conf = render_file(
-        beanstalk_apache_conf_template, beanstalk_settings)
-    with open('/etc/httpd/conf.d/beanstalk.conf', 'w') as f:
-        f.write(beanstalk_apache_conf)
-    local('echo "# Beanstalk Apache conf" > %s' % apache_conf_path())
-    local('chown beanstalk:beanstalk %s' % apache_conf_path())
+    print section_title('Configure Apache HTTP server')
+    if not os.path.exists('/etc/httpd/conf.d/beanstalk.conf'):
+        beanstalk_apache_conf_template = os.path.join(BEANSTALK_ROOT_PATH, 'static/beanstalk_apache.conf')
+        beanstalk_apache_conf = render_file(beanstalk_apache_conf_template, beanstalk_settings)
+        with open('/etc/httpd/conf.d/beanstalk.conf', 'w') as f:
+            f.write(beanstalk_apache_conf)
+    if not os.path.exists(apache_conf_path()):
+        local('echo "# Beanstalk Apache conf" > %s' % apache_conf_path())
+        local('chown beanstalk:beanstalk %s' % apache_conf_path())
     if not os.path.exists('/etc/httpd/conf.d/beanstalk_app.conf'):
         local('ln -s %s /etc/httpd/conf.d/beanstalk_app.conf' % apache_conf_path())
 
@@ -150,11 +174,11 @@ def setup(settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def create_app(project_name, settings_path=None, **settings_patches):
+def create_app(project_name, **settings_patches):
     """Create an app
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     env.project_name = project_name
     set_verbose_level()
 
@@ -237,7 +261,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
             ' ',
         ]
         sed_command = '\\\n'.join(commands)
-        local('sed -i.bak -e \'$a\\%s\' %s && rm -rf %s.bak' % (sed_command, apache_conf_path(), apache_conf_path()))
+        local(sed('$a\\{command}'.format(command=sed_command), apache_conf_path()))
         local('sudo /usr/sbin/httpd_reload')
 
     port_file_path = os.path.join(project_base_path(project_name), '%d.port' % uwsgi_port)
@@ -249,11 +273,11 @@ def create_app(project_name, settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def delete_app(project_name, settings_path=None, **settings_patches):
+def delete_app(project_name, **settings_patches):
     """Delete an app
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     set_verbose_level()
     env.project_name = project_name
 
@@ -278,17 +302,17 @@ def delete_app(project_name, settings_path=None, **settings_patches):
 
         local('rm -rf %s' % project_base)
 
-    local('sed -i.bak -e \'/# %s/,+7d\' %s && rm -rf %s.bak' % (project_name, apache_conf_path(), apache_conf_path()))
+    local(sed('/# {name}/,+7d'.format(name=project_name), apache_conf_path()))
     local('sudo /usr/sbin/httpd_reload')
 
 
 @task
 @beanstalk_role('server')
-def install_app(project_name, settings_path=None, **settings_patches):
+def install_app(project_name, **settings_patches):
     """Install an app (call when u updated the code)
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     set_verbose_level()
     env.project_name = project_name
 
@@ -341,11 +365,11 @@ def install_app(project_name, settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def reload_app(project_name, settings_path=None, **settings_patches):
+def reload_app(project_name, **settings_patches):
     """Update an app (call when u updated the code)
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     set_verbose_level()
     env.project_name = project_name
 
@@ -359,11 +383,11 @@ def reload_app(project_name, settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def build_venv(project_name, settings_path=None, **settings_patches):
+def build_venv(project_name, **settings_patches):
     """Build virtual env of target app.
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     set_verbose_level()
 
     # path
@@ -384,11 +408,11 @@ def build_venv(project_name, settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def app_info(project_name, settings_path=None, **settings_patches):
+def app_info(project_name, **settings_patches):
     """Get info of an app
     """
     settings_patches = eval_kwargs(settings_patches)
-    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    beanstalk_settings = load_beanstalk_settings(**settings_patches)
     set_verbose_level()
 
     # Gather information
@@ -438,11 +462,11 @@ def app_info(project_name, settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def list_apps(settings_path=None, **settings_patches):
+def list_apps(**settings_patches):
     """Get list of apps
     """
     settings_patches = eval_kwargs(settings_patches)
-    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    beanstalk_settings = load_beanstalk_settings(**settings_patches)
     set_verbose_level()
 
     apps = os.listdir(os.path.join(beanstalk_settings['BEANSTALK_STACK_BASE'], 'apps/'))
@@ -455,11 +479,11 @@ def list_apps(settings_path=None, **settings_patches):
 
 @task
 @beanstalk_role('server')
-def commit_app(project_name, settings_path=None, **settings_patches):
+def commit_app(project_name, **settings_patches):
     """Commit app
     """
     settings_patches = eval_kwargs(settings_patches)
-    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    load_beanstalk_settings(**settings_patches)
     env.project_name = project_name
     set_verbose_level()
 
@@ -470,3 +494,28 @@ def commit_app(project_name, settings_path=None, **settings_patches):
         local('git add -A && git commit -m "state at %s"' % datetime.datetime.now().strftime('%s'))
 
     run_hooked_actions('POST_COMMIT_ACTION')
+
+
+@task
+@beanstalk_role('server')
+def add_user(tmp_key_path):
+    """Add a user's SSH public key to beanstalk
+    """
+    with open(tmp_key_path, 'r') as f:
+        raw_key = f.read().strip()
+
+    new_key = 'no-port-forwarding,no-X11-forwarding,no-agent-forwarding {key}'.format(key=raw_key)
+
+    has_this_key = False
+    authorized_keys_path = os.path.expanduser('~/.ssh/authorized_keys')
+    with open(authorized_keys_path, 'r') as authorized_keys:
+        for authorized_key in authorized_keys:
+            if authorized_key.strip() == new_key:
+                has_this_key = True
+                break
+
+    if not has_this_key:
+        local('cp {0} {0}.bak && echo "{1}" >> {0} && rm -rf {0}.bak'.format(authorized_keys_path, new_key))
+
+    print green('User added.')
+
